﻿using System;
using System.Collections.Generic;
using UnityEngine;

public class PlacementSystem : MonoBehaviour
{
    [SerializeField] private int _maxGridX = default;
    [SerializeField] private int _maxGridY = default;
    [SerializeField] private Transform[] _worldSpaces = default;
    

    public GridSpace[,] BuildingGrid { get; private set; }
    public int MaxGridX => _maxGridX;
    public int MaxGridY => _maxGridY;

    public event EventHandler<BuildingChangedEventArgs> OnBuildingChanged;
    public event EventHandler<BuildingChangedEventArgs> OnBuildingDestroyed;

    private float _currentSecondCycle = 0;

    void Start()
    {
        // inefficent, who cares
        BuildingGrid = new GridSpace[_maxGridX, _maxGridY];
        if(_worldSpaces.Length < _maxGridX * _maxGridY)
        {
            throw new Exception("Not enough world transforms!");
        }
        int count = 0;
        for (int j = 0; j < _maxGridY; j++)
        {
            for (int i = 0; i < _maxGridX; i++)
            {
                BuildingGrid[i, j] = new GridSpace(i, j, _worldSpaces[count], 10);
                count++;
            }
        }
    }

    public int GetAliveBuildings()
    {
        int count = 0;
        for (int j = 0; j < _maxGridY; j++)
        {
            for (int i = 0; i < _maxGridX; i++)
            {
                if(BuildingGrid[i, j].IsAlive && BuildingGrid[i, j].CurrentBuilding)
                {
                    count++;
                }
            }
        }
        return count;
    }

    public int GetGridSpacesLeft()
    {
        int count = 0;
        for(int i = 0; i < -_maxGridX; i++)
        {
            for(int j = 0; j < _maxGridY; j++)
            {
                if(BuildingGrid[i, j].IsAlive)
                {
                    count++;
                }
            }
        }
        return count;
    }

    public bool PosOfFirstLivingGridSpace(out Vector2 pos)
    {
        pos = new Vector2();
        for (int j = 0; j < _maxGridY; j++)
        {
            for (int i = 0; i < _maxGridX; i++)
            {   
                if(BuildingGrid[i, j].IsAlive)
                {
                    pos.x = BuildingGrid[i, j].WorldSpace.position.x;
                    pos.y = BuildingGrid[i, j].WorldSpace.position.y;
                    return true;
                }
            }
        }
        return false;
    }

    public void PlaceBuilding(Building building, int x, int y)
    {
        // x determines tier
        // for development only
        if(x >= _maxGridX)
        {
            throw new Exception("Programmer error in value of grid x! X: " + x);
        }
        if(y >= _maxGridY)
        {
            throw new Exception("Programmer error in value of grid y! Y: " + y);
        }
        // todo play animation based on building changing
        // send out event that building changed?
        if(BuildingGrid[x, y].IsAlive)
        {
            if(BuildingGrid[x, y].CurrentBuilding)
            {
                OnBuildingDestroyed?.Invoke(this, new BuildingChangedEventArgs
                {
                    building = building,
                    x = x,
                    y = y
                });
            }
            BuildingGrid[x, y].CurrentBuilding = building;
            building.transform.position = BuildingGrid[x, y].WorldSpace.position;
            OnBuildingChanged?.Invoke(this, new BuildingChangedEventArgs
            {
                building = building,
                x = x,
                y = y
            });
        }
    }

    public void RepairBrokenBuildings(int numBuilders, int repairRate)
    {
        for (int j = 0; j < _maxGridY; j++)
        {
            for (int i = 0; i < _maxGridX; i++)
            {
                if(BuildingGrid[i, j].IsAlive)
                {
                    Building build = BuildingGrid[i, j].CurrentBuilding;
                    if (build && build.Type != null && build.NeedsReapirs)
                    {
                        build.ModifyRepairPoints(numBuilders * repairRate);
                    }
                }
            }
        }
    }

    public void AttackSpace(int x, int y, int attack)
    {
        Building build = BuildingGrid[x, y].CurrentBuilding;
        if(build && build.IsAlive)
        {
            build.ModifyRepairPoints(-attack);
            if(!build.IsAlive)
            {
                OnBuildingDestroyed?.Invoke(this, new BuildingChangedEventArgs
                {
                    building = build,
                    x = x,
                    y = y
                });
            }
        }
        else
        {
            // attack space
            BuildingGrid[x, y].DecreaseHP(attack);
            if(!BuildingGrid[x, y].IsAlive)
            {
                // uh oh, space destroyed
                // play associated animation?
            }
        }
    }
}
