﻿using System;
using System.Collections.Generic;
using UnityEngine;

public class Enemy : MonoBehaviour
{
    [SerializeField] private SpriteRenderer _renderer = default;
    [SerializeField] private Transform[] _bouncePoints = default;
    [SerializeField] private PlacementSystem _placeSys = default;
    [SerializeField] private PeopleSystem _peopleSys = default;
    [SerializeField] private float _moveSpeed = default;
    [SerializeField] private float _yMoveDown = default;
    [SerializeField] private float _attackEveryXSeconds = default;
    [SerializeField] private int _attackStrength = default;
    [SerializeField] private float _minEngageDistance = default;

    public EnemyType Type;

    public int PosInEnemyList { get; set; }

    public bool IsAlive => _currentHP > 0;

    public event EventHandler<EntityKilledArgs> EnemyKilled;

    private int _currentHP = default;

    private int _currDirection = -1;

    private float _currentSecondTime = 0;

    private void Start()
    {
        _currentHP = Type.BaseHP;
        if(Type.Sprite)
        {
            _renderer.sprite = Type.Sprite;
        }
    }

    public void CreateEnemySprite(Transform[] bouncePoints,
        PlacementSystem placeSys, PeopleSystem peopleSys)
    {
        _bouncePoints = bouncePoints;
        _placeSys = placeSys;
        _peopleSys = peopleSys;
    }

    public void TickAttack()
    {
        // keep going in the correct direction
        Tuple<bool, GridSpace> possibleBuilding = IsThereBuildingNearby();
        if(possibleBuilding.Item1)
        {
            // go attack
            GridSpace build = possibleBuilding.Item2;
            if (Vector2.Distance(transform.position, build.WorldSpace.position) > 0.01)
            {
                transform.Translate(new Vector2(_moveSpeed * _currDirection, 0)
                * Time.deltaTime);
            }
            else
            {
                _currentSecondTime += Time.deltaTime;
                if(_currentSecondTime >= _attackEveryXSeconds)
                {
                    _currentSecondTime = 0;
                    if (GuardsNearby(out Guard closestGuard))
                    {
                        closestGuard.GetAttacked(_attackStrength);
                    }
                    else
                    {
                        _placeSys.AttackSpace(build.GridX, build.GridY, _attackStrength);
                    }
                }
            }
        }
        else
        {
            _currentSecondTime = 0;
            transform.Translate(new Vector2(_moveSpeed * _currDirection, 0) 
                * Time.deltaTime);
            for (int i = 0; i < _bouncePoints.Length; i++)
            {
                if (Vector2.Distance(_bouncePoints[i].position, transform.position) < 0.01)
                {
                    // change direction
                    _currDirection = _currDirection * -1;
                    transform.position = new Vector2(transform.position.x, transform.position.y + _yMoveDown);
                }
            }
        }
    }

    public void TakeDamage(int amount)
    {
        _currentHP -= amount;
        if(_currentHP <= 0)
        {
            // todo we're dead now. RIP
            EnemyKilled?.Invoke(this, new EntityKilledArgs
            {
                listPos = PosInEnemyList
            });
        }
    }

    public void Kill()
    {
        // kill them at end of cycle
        EnemyKilled?.Invoke(this, new EntityKilledArgs
        {
            listPos = PosInEnemyList
        });
    }

    // todo put this somewehre :V
    private Tuple<bool, GridSpace> IsThereBuildingNearby()
    {
        Tuple<bool, GridSpace> value = new Tuple<bool, GridSpace>(false, null);
        GridSpace[,] spaces = _placeSys.BuildingGrid;
        int x = -1;
        int y = -1;
        for (int i = 0; i < _placeSys.MaxGridX; i++)
        {
            for (int j = 0; j < _placeSys.MaxGridY; j++)
            {
                if (spaces[i, j].IsAlive &&
                    Vector2.Distance(spaces[i, j].WorldSpace.position
                    , transform.position) < _minEngageDistance)
                {
                    if (x > -1 && y > -1)
                    {
                        if (Vector2.Distance(spaces[x, y].WorldSpace.position, transform.position)
                            > Vector2.Distance(spaces[i, y].WorldSpace.position, transform.position))
                        {
                            x = i;
                            y = j;
                        }
                    }
                    else
                    {
                        x = i;
                        y = j;
                    }
                }
            }
        }
        if (x > -1 && y > -1)
        {
            value = new Tuple<bool, GridSpace>(true, spaces[x, y]);
        }
        return value;
    }

    private bool GuardsNearby(out Guard guard)
    {
        bool value = false;
        guard = null;
        List<Guard> guards = new List<Guard>(_peopleSys.CurrentGuards);
        int guardLocation = -1;
        for(int i = 0; i < guards.Count; i++)
        {
            if(Vector2.Distance(guards[i].transform.position, transform.position) < _minEngageDistance)
            {
                if(guardLocation > -1)
                {
                    if(Vector2.Distance(guards[guardLocation].transform.position, transform.position) >
                        Vector2.Distance(guards[i].transform.position, transform.position))
                    {
                        guardLocation = i;
                    }
                }
                else
                {
                    guardLocation = i;
                }
            }
        }

        if(guardLocation > - 1)
        {
            guard = guards[guardLocation];
            value = true;
        }

        return value;
    }
}