﻿using System;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
public class GameState : MonoBehaviour
{
    [SerializeField] private MoraleDefenseSystem _moralDefenseSys = default;
    [SerializeField] private PlacementSystem _placementSys = default;
    [SerializeField] private CycleSystem _cycleSystem = default;
    [SerializeField] private InGameEventSystem _iges = default;
    [SerializeField] private PeopleSystem _ps = default;
    [SerializeField] private EnemySystem _es = default;
    [SerializeField] private PlaceTypeButton[] _placeTypeButtons = default;
    [SerializeField] private SelectTypeButton[] _selectTypeButtons = default;
    [SerializeField] private Button[] _increasePopButtons = default;
    [SerializeField] private EventPopupSystem _popSys = default;
    [SerializeField] private InGameEvent _introEvent = default;
    [SerializeField] private InGameEvent _outroEvent = default;

    public int ResourceCount { get; private set;  }

    public event EventHandler<EventArgs> SliderValuesChanged;

    private bool _cycleRunning = true;

    private int _guardsAtAttackStart;
    private int _buildingsAtAttackStart;
    private int _defenseAtAttackStart;
    private int _gridSpacesLeftAtAttackStart;

    private float _currentSecondTime = 0;

    private float _currentAttckCycleSecond = 0;

    public void GameStarted()
    {
        _cycleRunning = true;
    }

    public void GameStopped()
    {
        _cycleRunning = false;
    }

    void Start()
    {
        _placementSys.OnBuildingChanged += (sender, built) =>
        {
            // assume that we have ensured that we can't get below the resource cost
            ResourceCount -= built.building.Type.ResourceCost;
            _moralDefenseSys.ModifyDefense(built.building.Type.DefenseValue);
            _moralDefenseSys.BuildingMoraleGain += built.building.Type.MoraleGen;
            SliderValuesChanged?.Invoke(this, EventArgs.Empty);
        };

        _popSys.SetEventClosed(() =>
        {
            _cycleRunning = true;
        });

        _placementSys.OnBuildingDestroyed += (sender, built) =>
        {
            _moralDefenseSys.ModifyDefense(-built.building.Type.DefenseValue);
            _moralDefenseSys.BuildingMoraleGain -= built.building.Type.MoraleGen;
            SliderValuesChanged?.Invoke(this, EventArgs.Empty);
        };

        _cycleSystem.OnCycleChanged += (sender, cycleInfo) =>
        {
            SliderValuesChanged?.Invoke(this, EventArgs.Empty);
            for (int i = 0; i < _placeTypeButtons.Length; i++)
            {
                _placeTypeButtons[i].gameObject.GetComponent<Button>().interactable = (cycleInfo.isPeaceful);
            }
            for(int i = 0; i < _selectTypeButtons.Length; i++)
            {
                _selectTypeButtons[i].gameObject.GetComponent<Button>().interactable = (cycleInfo.isPeaceful);
            }
            for (int i = 0; i < _increasePopButtons.Length; i++)
            {
                _increasePopButtons[i].interactable = (cycleInfo.isPeaceful);
            }
            if (cycleInfo.isPeaceful)
            {
                _ps.EndAttackCycle();
                _es.EndAttackCycle();
                _ps.BeginPeacefulCycle();

                // todo announce peaceful cycle, get an event from the end of the last cycle
                //Tuple<InGameEvent, float> eve = _iges.GetEventBasedOnPerformance(
                //    _moralDefenseSys.Defense / _defenseAtAttackStart,
                //    50 / _guardsAtAttackStart,
                //    _moralDefenseSys.CurrentMorale / _moralDefenseSys.MaxMorale,
                //    _placementSys.GetAliveBuildings() / _buildingsAtAttackStart,
                //     _placementSys.GetGridSpacesLeft() / _gridSpacesLeftAtAttackStart);
                //CallEventPopUp(eve.Item1);
                // display event
            }
            else
            {
                _ps.EndPeacefulCycle();
                _es.BeginAttackCycle();
                _ps.BeginAttackCycle();
                // todo get event based on priorities
                //InGameEvent eve = _iges.GetRandomEvent();               
                //CallEventPopUp(eve);
                _guardsAtAttackStart = _ps.CurrentGuardCount;
                _buildingsAtAttackStart = _placementSys.GetAliveBuildings();
                _defenseAtAttackStart = _moralDefenseSys.Defense;
                _gridSpacesLeftAtAttackStart = _placementSys.GetGridSpacesLeft();
                
            }
        };
        CallEventPopUp(_introEvent);
    }

    private void CallEventPopUp(InGameEvent gameEvent)
    {
        _cycleRunning = false;
        _popSys.EventActive(gameEvent);
    }

    private void Update()
    {
        if(_cycleRunning)
        {
            
            if(_cycleSystem.IsPeaceful)
            {
                _moralDefenseSys.TickPeacefulCycle();
                _ps.TickPeacefulCycle();
                // different animations?
                // repair damaged buildings, taking away resources
                _currentSecondTime += Time.deltaTime;
                if (_currentSecondTime >= 1.0f)
                {
                    _currentSecondTime = 0;
                    ResourceCount += (int)(_moralDefenseSys.NumPeople * .50f);
                }
            }
            else
            {
                // enemies, people systems
                _ps.TickAttackCycle();
                _es.TickAttackCycle();
                TickAttackCycle();
            }
            _moralDefenseSys.TickCycle();
            _cycleSystem.TickCycle();
            if(_moralDefenseSys.CurrentMorale <= 0)
            {
                _cycleRunning = false;
                CallEventPopUp(_outroEvent);
                _popSys.SetEventClosed(() =>
                {
                    Application.Quit(0);
                });
                // pop up game over
            }
        }
    }

    private void TickAttackCycle()
    {
        _currentAttckCycleSecond += Time.deltaTime;
        if (_currentAttckCycleSecond >= 3.0f)
        {
            _currentAttckCycleSecond = 0;
            List<Enemy> enemies = new List<Enemy>(_es.CurrentEnemies);
            for (int i = 0; i < enemies.Count; i++)
            {
                if (enemies[i].IsAlive)
                {
                    enemies[i].TakeDamage((int)(_moralDefenseSys.Defense * .10));
                }
            }
        }
    }
}
