﻿using System;
using System.Collections.Generic;
using UnityEngine;

public class PeopleSystem : MonoBehaviour
{
    [SerializeField][Range(5, 20)] private int _increaseByEachPress;
    [SerializeField][Range(5, 100)] private float _spacingBetweenGuards;
    [SerializeField] private int _builderRepairRate;
    [SerializeField] private int _medicMoraleRaiseRate;
    [SerializeField] private Guard _protoGuard;
    [SerializeField] private MoraleDefenseSystem _moraleDefenseSys;
    [SerializeField] private PlacementSystem _placeSys;
    public int PercentMedics { get; private set; } = 0;
    public int PercentGuards { get; private set; } = 0;
    public int PercentBuilders { get; private set; } = 0;
    public int PercentIdles { get; private set; } = 100;

    public IEnumerable<Guard> CurrentGuards => _guardsThisAttackCycle;
    public int CurrentGuardCount => _guardsThisAttackCycle.Count;

    public event EventHandler<EventArgs> GuardCountChanged;

    private List<Guard> _guardsThisAttackCycle = new List<Guard>();

    private int _currentNumGuards;

    private float _currentSecondTime = 0;

    public void IncreaseMedics()
    {
        if(PercentMedics < 100)
        {
            PercentMedics += _increaseByEachPress;
            // prioritise Idles
            if(PercentIdles > 0)
            {
                PercentIdles -= _increaseByEachPress;
                _moraleDefenseSys.ModifyMaxMorale(
                    -Mathf.Abs(_moraleDefenseSys.NumPeople
                    * (_increaseByEachPress / 100)));
            }
            else
            {
                // check which one is the highest
                if(PercentGuards > PercentBuilders)
                {
                    PercentGuards -= _increaseByEachPress;
                    GuardCountChanged?.Invoke(this, EventArgs.Empty);
                }
                else
                {
                    PercentBuilders -= _increaseByEachPress;
                }
            }
        }
    }

    public void IncreaseGuards()
    {
        if (PercentGuards < 100)
        {
            PercentGuards += _increaseByEachPress;
            GuardCountChanged?.Invoke(this, EventArgs.Empty);
            // prioritise Idles
            if (PercentIdles > 0)
            {
                PercentIdles -= _increaseByEachPress;
                _moraleDefenseSys.ModifyMaxMorale(
                    -Mathf.Abs(_moraleDefenseSys.NumPeople
                    * (_increaseByEachPress / 100)));
            }
            else
            {
                // check which one is the highest
                if (PercentMedics > PercentBuilders)
                {
                    PercentMedics -= _increaseByEachPress;
                }
                else
                {
                    PercentBuilders -= _increaseByEachPress;
                }
            }
        }
    }

    public void IncreaseBuilders()
    {
        if (PercentBuilders < 100)
        {
            PercentBuilders += _increaseByEachPress;
            // prioritise Idles
            if (PercentIdles > 0)
            {
                PercentIdles -= _increaseByEachPress;
                _moraleDefenseSys.ModifyMaxMorale(
                    -Mathf.Abs(_moraleDefenseSys.NumPeople 
                    * (_increaseByEachPress / 100)));
            }
            else
            {
                // check which one is the highest
                if (PercentMedics > PercentGuards)
                {
                    PercentMedics -= _increaseByEachPress;
                }
                else
                {
                    GuardCountChanged?.Invoke(this, EventArgs.Empty);
                    PercentGuards -= _increaseByEachPress;
                }
            }
        }
    }

    public void IncreaseIdles()
    {
        // one difference - increase max morale by these
        // and lower it when these are lowered
        if(PercentIdles < 100)
        {
            PercentIdles += _increaseByEachPress;
            _moraleDefenseSys.ModifyMaxMorale(
                Mathf.Abs(_moraleDefenseSys.NumPeople
                    * (_increaseByEachPress / 100)));
            // take it away from one of the others
            if(PercentGuards > 0)
            {
                GuardCountChanged?.Invoke(this, EventArgs.Empty);
                PercentGuards -= _increaseByEachPress;
            }
            else if(PercentMedics > 0)
            {
                PercentMedics -= _increaseByEachPress;
            }
            else if(PercentBuilders > 0)
            {
                PercentBuilders -= _increaseByEachPress;
            }       
        } 
    }

    public void BeginPeacefulCycle()
    {

    }

    public void TickPeacefulCycle()
    {  
        _currentSecondTime += Time.deltaTime;
        _moraleDefenseSys.PopulationMoraleDrain = _moraleDefenseSys.NumPeople; // todo multiple by something
        if (_currentSecondTime >= 1.0f)
        {
            // do repairs on buildings
            // raise our current morale based on medics
            _currentSecondTime = 0;
            int numBuilders = Mathf.Abs(_moraleDefenseSys.NumPeople * (PercentBuilders) / 100);
            int numMedics = Mathf.Abs(_moraleDefenseSys.NumPeople * (PercentMedics) / 100);
            _moraleDefenseSys.MedicMoraleGain = (numMedics * _medicMoraleRaiseRate);
            _placeSys.RepairBrokenBuildings(numBuilders, _builderRepairRate);
        }
    }

    public void EndPeacefulCycle()
    {

    }

    public void BeginAttackCycle()
    {
        // spawn the percentage num of guards we need
        int numGuards = Mathf.Abs(_moraleDefenseSys.NumPeople * (PercentGuards / 100));
        if (_placeSys.PosOfFirstLivingGridSpace(out Vector2 pos))
        {
            float currentX = pos.x;
            for (int i = 0; i < numGuards; i++)
            {
                SpawnGuard(currentX, pos.y);
                currentX += _spacingBetweenGuards;   
            }
        }
        else
        {
            // we should have lost by now?
            throw new System.Exception("We should have lost - how did our morale survive" +
                " with no spaces left?");
        }
    }

    public void TickAttackCycle()
    {
        for(int i = 0; i < _guardsThisAttackCycle.Count; i++)
        {
            _guardsThisAttackCycle[i].TickAttack();
        }
    }

    public void EndAttackCycle()
    {
        // unsure
    }

    private void SpawnGuard(float xPos, float yPos)
    {
        Guard guard = Instantiate(_protoGuard);
        guard.transform.position = new Vector3(xPos, yPos);
        guard.PosInGuardsList = _guardsThisAttackCycle.Count;
        guard.GuardKilled += (sender, info) =>
        {
            _guardsThisAttackCycle[info.listPos].gameObject.SetActive(false);
            _currentNumGuards--;
            if(_currentNumGuards <= 0)
            {
                AllGuardsDead();
            }
        };
        _currentNumGuards++;
        _guardsThisAttackCycle.Add(guard);
    }

    private void AllGuardsDead()
    {
        for(int i = 0; i < _guardsThisAttackCycle.Count; i++)
        {
            Destroy(_guardsThisAttackCycle[i]);
        }
    }
}
